/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Various message channel profile related utility methods.
 */
Ext.define( 'QDesk.controller.command.message.channel.MessageChannelProfileUtils', {
  singleton : true,

  requires : [
    'QDesk.enums.MessageChannel',
    'QDesk.view.main.message.channel.email.EmailProfilePanel',
    'QDesk.view.main.message.channel.im_aim.ImAimProfilePanel',
    'QDesk.view.main.message.channel.im_gtalk.ImGTalkProfilePanel',
    'QDesk.view.main.message.channel.im_icq.ImIcqProfilePanel',
    'QDesk.view.main.message.channel.im_skype.ImSkypeProfilePanel',
    'QDesk.view.main.message.channel.im_xmpp.ImXmppProfilePanel',
    'QDesk.view.main.message.channel.im_yahoo.ImYahooProfilePanel',
    'QDesk.view.main.message.channel.web_service.WebServiceProfilePanel'
  ],

  /**
   * Returns the message channel profiles window.
   * @return the window.
   */
  getProfilesWindow : function ()
  {
    var window = Ext.ComponentQuery.query( 'MessageChannelProfilesWindow' )[0];

    if ( !window )
      Ext.Error.raise( 'Message channel profiles window not found.' );

    return window;
  },


  /**
   * Returns the message channel profiles tree panel.
   * @return the tree panel.
   */
  getProfilesTreePanel : function ()
  {
    var window = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesWindow();
    return window.down( '#messageChannelProfilesTreePanel' );
  },


  /**
   * Returns the message channel profiles edit panel.
   * @return the edit panel.
   */
  getEditPanel : function ()
  {
    var window = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesWindow();
    return window.down( '#editPanel' );
  },


  /**
   * Adds the specified form panel to the edit panel and makes the panel active (edit panel uses card layout).
   * @param panel a panel to add.
   */
  addAndShowPanel : function ( panel )
  {
    var editPanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getEditPanel();
    editPanel.add( panel );
    editPanel.getLayout().setActiveItem( panel );
  },


  /**
   * Returns the node ID of the specified profile.
   * @param profileId a profile ID.
   * @return the node ID of the specified profile.
   */
  getProfileNodeId : function ( profileId )
  {
    return 'profile-' + profileId;
  },


  /**
   * Returns the name of the profile with the specified ID.
   * @param profileId a profile ID.
   */
  getProfileName : function ( profileId )
  {
    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfileNodeId( profileId );
    var profileNode = store.getNodeById( nodeId );
    return profileNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified profile, or null if not found.
   * @param profileId a profile ID.
   * @return the parent folder ID, or null.
   */
  getProfileParentFolderId : function ( profileId )
  {
    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfileNodeId( profileId );
    var profileNode = store.getNodeById( nodeId );
    return profileNode ? profileNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the node ID of the specified folder.
   * @param folderId a folder ID.
   * @return the node ID of the specified folder.
   */
  getFolderNodeId : function ( folderId )
  {
    return folderId == QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_FOLDER_ID ?
        QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_ID : 'folder-' + folderId;
  },


  /**
   * Returns the name of the folder with the specified ID.
   * @param folderId a folder ID.
   */
  getFolderName : function ( folderId )
  {
    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified folder, or null if not found.
   * @param folderId a folder ID.
   * @return the parent folder ID, or null.
   */
  getFolderParentFolderId : function ( folderId )
  {
    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode ? folderNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the {@link QDesk.enums.MessageChannel} for the specified profile ID.
   * @param profileId a profile ID.
   * @return the {@link QDesk.enums.MessageChannel} of the specified profile.
   */
  getProfileMessageChannel : function ( profileId )
  {
    var treePanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getProfileNodeId( profileId );
    var profileNode = store.getNodeById( nodeId );
    return profileNode.get( 'messageChannel' );
  },


  cleanupEditPanel : function ()
  {
    var editPanel = QDesk.controller.command.message.channel.MessageChannelProfileUtils.getEditPanel();


    // remove all panels (profile editors) from the edit panel (right panel), but the infoPanel
    Ext.each( editPanel.getLayout().getLayoutItems(), function ( panel )
    {
      if ( panel.getItemId() == 'infoPanel' )
      {
        // edit panel uses card layout, so we activate the info panel
        editPanel.getLayout().setActiveItem( panel );
      }
      else
      {
        editPanel.remove( panel );
      }
    } );
  },


  /**
   * Creates and returns an empty profile panel for the specified message channel.
   * @param messageChannel a message channel.
   * @return the created profile panel.
   */
  createEmptyProfilePanel : function ( messageChannel )
  {
    switch ( messageChannel )
    {
      case QDesk.enums.MessageChannel.EMAIL:
        return Ext.widget( 'EmailProfilePanel' );

      case QDesk.enums.MessageChannel.IM_AIM:
        return Ext.widget( 'ImAimProfilePanel' );

      case QDesk.enums.MessageChannel.IM_GTALK:
        return Ext.widget( 'ImGTalkProfilePanel' );

      case QDesk.enums.MessageChannel.IM_ICQ:
        return Ext.widget( 'ImIcqProfilePanel' );

      case QDesk.enums.MessageChannel.IM_SKYPE:
        return Ext.widget( 'ImSkypeProfilePanel' );

      case QDesk.enums.MessageChannel.IM_XMPP:
        return Ext.widget( 'ImXmppProfilePanel' );

      case QDesk.enums.MessageChannel.IM_YAHOO:
        return Ext.widget( 'ImYahooProfilePanel' );

      case QDesk.enums.MessageChannel.WEB_SERVICE:
        return Ext.widget( 'WebServiceProfilePanel' );

      default:
        Ext.Error.raise( 'Unsupported message channel: ' + messageChannel );
    }
  }
} );
