/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the view panel for the Quartz scheduler described by the specified
 * connection description configuration object.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProduct',
    'QDesk.controller.command.scheduler.quartz.RefreshActiveTab',
    'QDesk.controller.command.scheduler.StartScheduler',
    'QDesk.controller.command.scheduler.PauseScheduler',
    'QDesk.controller.command.scheduler.StopScheduler',
    'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab',
    'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab',
    'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab',
    'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab',
    'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab'
  ],

  /*
   * Required connection ID configuration attribute.
   */
  connectionId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[0];
    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( me.connectionId );

    // if there is no tab panel for the scheduler, create a new one and add it to the mainTabPanel, otherwise
    // just activate the existing tab panel
    if ( schedulerTab == null )
    {
      QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.maskMsg' ) );

      // determine if the JVM Agent is available for the specified connection
      Ext.Ajax.request( {
        url : QDesk.util.constant.ActionUrl.agentActionAvailability,
        params : {
          connectionId : me.connectionId
        },
        callback : function ( opts, success, resp )
        {
          QDesk.util.UxUtils.unmaskBody();

          if ( success )
          {
            var json = Ext.decode( resp.responseText );
            if ( json.success )
            {
              if ( json.data.jvmAgentAvailable )
              {
                /* Start of QuartzDesk JVM Agent vs QuartzDesk app version check */
                var jvmAgentVersion = json.data.jvmAgentVersion;
                var productVersion = QDesk.util.license.LicensedProduct.productVersion;

                if ( jvmAgentVersion.major == productVersion.major )
                {
                  // versions match
                  me._openSchedulerTabPanel( mainTabPanel, true );
                }
                else
                {
                  QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentVersionsMismatchMsg',
                      QDesk.util.FormatUtils.formatProductVersion( jvmAgentVersion ),
                      QDesk.util.FormatUtils.formatProductVersion( productVersion ) ),
                      function ()
                      {
                        me._openSchedulerTabPanel( mainTabPanel, false );
                      } );

                }
                /* End of QuartzDesk JVM Agent vs QuartzDesk app version check */
              }
              else
              {
                // QuartzDesk JVM Agent not available
                if ( QDesk.Config.get( 'warning.jvmAgentNotAvailable' ) )
                {
                  QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentNotAvailableMsg' ),
                      function ()
                      {
                        me._openSchedulerTabPanel( mainTabPanel, false );
                      } );
                }
                else
                {
                  me._openSchedulerTabPanel( mainTabPanel, false );
                }
              }
            }
            else
            {
              // Server side error
              QDesk.util.UxUtils.showServerError( json );
            }
          }
          else
          {
            // Ajax error
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.errorMsg' ) );
          }
        }
      } );
    }
    else
    {
      // make the panel active
      mainTabPanel.setActiveTab( schedulerTab );
    }
  },


  /**
   * Opens the Quartz scheduler tab and adds it to the specified main tab panel.
   * @param mainTabPanel the main tab panel.
   * @param jvmAgentAvailable a flag indicating if the QuartzDesk JVM Agent is available.
   */
  _openSchedulerTabPanel : function ( mainTabPanel, jvmAgentAvailable )
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.maskMsg' ) );

    // deferred so that the mask message gets displayed
    Ext.defer( function ()
    {
      var me = this;

      // open a new Quartz scheduler tab panel and add it to the main tab panel
      schedulerTab = mainTabPanel.add( {
        xtype : 'QuartzSchedulerTab',
        itemId : QDesk.controller.command.connection.ConnectionUtils.getSchedulerTabItemId( me.connectionId ),
        title : QDesk.controller.command.connection.ConnectionUtils.getConnectionName( me.connectionId ),
        qdesk : {
          connectionId : me.connectionId,
          jvmAgentAvailable : jvmAgentAvailable
        }
      } );

      mainTabPanel.setActiveTab( schedulerTab );

      me._initializeSchedulerTabPanel( schedulerTab, me.connectionId );

      me.unmask();
    }, 100, me );
  },


  /**
   * Initializes the scheduler tab panel by binding all UI panel controls with the logic.
   * @param tabPanel a Quartz scheduler tab panel.
   */
  _initializeSchedulerTabPanel : function ( tabPanel )
  {
    var me = this;

    var refreshButton = tabPanel.down( '#schedulerTabToolBar > #refreshButton' );
    refreshButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshActiveTab', {
        connectionId : me.connectionId
      } ).execute();
    } );

    var startButton = tabPanel.down( '#schedulerTabToolBar > #startSchedulerButton' );
    startButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.StartScheduler', {
        connectionId : me.connectionId
      } ).execute();
    } );

    var pauseButton = tabPanel.down( '#schedulerTabToolBar > #pauseSchedulerButton' );
    pauseButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.PauseScheduler', {
        connectionId : me.connectionId
      } ).execute();
    } );

    var stopButton = tabPanel.down( '#schedulerTabToolBar > #stopSchedulerButton' );
    stopButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.StopScheduler', {
        connectionId : me.connectionId
      } ).execute();
    } );

    me._initializeSubTabs( tabPanel );
  },


  /**
   * Initializes sub tabs of the specified scheduler tab.
   * @param tabPanel a scheduler tab (the main tab).
   */
  _initializeSubTabs : function ( tabPanel )
  {
    var me = this;

    var jobsTab = tabPanel.down( '#jobsTab' );
    jobsTab.on( 'activate', function ( jobsTab )
    {
      /* If the jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !jobsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var triggersTab = tabPanel.down( '#triggersTab' );
    triggersTab.on( 'activate', function ( triggersTab )
    {
      /* If the triggers tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !triggersTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var infoTab = tabPanel.down( '#infoTab' );
    infoTab.on( 'activate', function ( infoTab )
    {
      /* If the info tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !infoTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var schedulerExecHistoryTab = tabPanel.down( '#schedulerExecHistoryTab' );
    schedulerExecHistoryTab.on( 'activate', function ( schedulerExecHistoryTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerExecHistoryTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab', {
          connectionId : me.connectionId
        } ).execute();


        Ext.create( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var schedulerStatisticsTab = tabPanel.down( '#schedulerStatisticsTab' );
    schedulerStatisticsTab.on( 'activate', function ( schedulerStatisticsTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerStatisticsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    var schedulerNotificationsTab = tabPanel.down( '#schedulerNotificationsTab' );
    schedulerNotificationsTab.on( 'activate', function ( schedulerNotificationsTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerNotificationsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } );


    /* Initially selected tab (this could be made configurable...) */
    tabPanel.setActiveTab( jobsTab );

    // setActiveTab does not fire the activate event if the tab is already active so we need to
    // simulate the active event...

    /* If the jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
    if ( !jobsTab.getInitialized() )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
        connectionId : me.connectionId
      } ).execute();
    }

    Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshActiveTab', {
      connectionId : me.connectionId
    } ).execute();
  }
} );
