/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Trigers tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.SaveChart',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator',
    'QDesk.controller.command.scheduler.quartz.ResumeTrigger',
    'QDesk.controller.command.scheduler.quartz.PauseTrigger',
    'QDesk.controller.command.scheduler.quartz.DeleteTrigger',
    'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule',
    'QDesk.enums.scheduler.quartz.TriggerGroupState',
    'QDesk.enums.scheduler.quartz.TriggerState',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var triggersTab = QDesk.controller.QuartzControllerUtils.getTriggersTab( me.connectionId );

    me._bindEventHandlers( triggersTab );

    triggersTab.setInitialized( true );
  },


  /**
   * Binds events from the triggers tab to their respective handlers.
   */
  _bindEventHandlers : function ( triggersTab )
  {
    var me = this;
    me._bindEventHandlersTriggersGrid( triggersTab );

    // individual views
    me._bindEventHandlersHistoryView( triggersTab );
    me._bindEventHandlersStatisticsView( triggersTab );
    me._bindEventHandlersNotificationsView( triggersTab );
  },


  /**
   * Binds the triggers grid events to their handlers.
   */
  _bindEventHandlersTriggersGrid : function ( triggersTab )
  {
    var me = this;

    var triggersGrid = triggersTab.down( '#triggersGrid' );

    // refresh button
    triggersGrid.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me._refreshTriggersGrid( me.connectionId );
        } );

    var resumeTriggerButton = triggersGrid.down( '#resumeTriggerButton' );
    resumeTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[0];
      var trigger = Ext.create( 'QDesk.model.quartz.Trigger', triggerWithJobDetail.get( 'trigger' ) );
      me._resumeTrigger( trigger );
    }, me );

    var pauseTriggerButton = triggersGrid.down( '#pauseTriggerButton' );
    pauseTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[0];
      var trigger = Ext.create( 'QDesk.model.quartz.Trigger', triggerWithJobDetail.get( 'trigger' ) );
      me._pauseTrigger( trigger );
    }, me );

    var deleteTriggerButton = triggersGrid.down( '#deleteTriggerButton' );
    deleteTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[0];
      var trigger = Ext.create( 'QDesk.model.quartz.Trigger', triggerWithJobDetail.get( 'trigger' ) );
      me._deleteTrigger( trigger );
    }, me );

    var filterByTriggerGroupCombo = triggersGrid.down( '#filterByTriggerGroupCombo' );
    filterByTriggerGroupCombo.on( 'change', function ( combo, newValue )
    {
      if ( newValue == null )
      {
        resumeTriggerGroupButton.disable();
        pauseTriggerGroupButton.disable();
      }
      else
      {
        // based on the selected trigger group state enable either the resume, or pause button
        var triggerGroup = filterByTriggerGroupCombo.getStore().findRecord( 'name', newValue );
        var triggerGroupState = triggerGroup.get( 'state' );

        switch ( triggerGroupState )
        {
          case QDesk.enums.scheduler.quartz.TriggerGroupState.NORMAL:
            resumeTriggerGroupButton.disable();
            pauseTriggerGroupButton.enable();
            break;

          case QDesk.enums.scheduler.quartz.TriggerGroupState.PAUSED:
            resumeTriggerGroupButton.enable();
            pauseTriggerGroupButton.disable();
            break;

          default:
            Ext.Error.raise( 'Unexpected trigger group state: ' + triggerGroupState );
        }
      }

      // apply, or clear the filter and load the first page
      triggersGrid.getStore().loadPage( 1, {
        filters : newValue == null ? [] :
            [Ext.create( 'Ext.util.Filter', { property : 'trigger.group.name', value : newValue } )]
      } );
    } );

    var resumeTriggerGroupButton = triggersGrid.down( '#resumeTriggerGroupButton' );
    resumeTriggerGroupButton.on( 'click', function ()
    {
      var triggerGroupName = filterByTriggerGroupCombo.getValue();
      me._resumeTriggerGroup( triggerGroupName );
    }, me );

    var pauseTriggerGroupButton = triggersGrid.down( '#pauseTriggerGroupButton' );
    pauseTriggerGroupButton.on( 'click', function ()
    {
      var triggerGroupName = filterByTriggerGroupCombo.getValue();
      me._pauseTriggerGroup( triggerGroupName );
    }, me );

    // event fired by the actioncolumn in the TriggersGrid
    triggersGrid.down( '#triggerJobDataMapColumn' ).on( 'requestViewJobDataMap',
        function ( jobDataMap, effectiveJobDataMap )
        {
          me._viewJobDataMap( jobDataMap, effectiveJobDataMap );
        } );

    triggersGrid.getStore().on( 'beforeload', function ( store, operation )
    {
      triggersGrid.down( '#refreshButton' ).disable();
      filterByTriggerGroupCombo.disable();

      /*
       * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
       * Therefore we need to force-fire this event by clearing the selection.
       */
      triggersGrid.getSelectionModel().deselectAll();
    } );

    triggersGrid.getStore().on( 'load', function ( triggerStore, records, successful, operation )
    {
      triggersGrid.down( '#refreshButton' ).enable();
      if ( successful )
      {
        if ( records.length > 0 )
          filterByTriggerGroupCombo.enable();
      }
      else
      {
        triggerStore.removeAll();

        QDesk.util.UxUtils.showErrorDialog(
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingTriggersMsg' ) );
      }
    } );

    triggersGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
    {
      if ( selectedRecords.length == 0 )
      {
        me._onTriggerUnselected( triggersTab );
      }
      else
      {
        me._onTriggerSelected( selectedRecords[0], triggersTab );
      }
    } );
  },


  /**
   * Binds the trigger execution history panel events to their handlers.
   */
  _bindEventHandlersHistoryView : function ( triggersTab )
  {
    var me = this;

    var historyView = triggersTab.down( '#historyView' );
    historyView.on( 'activate', function ()
    {
      me._refreshHistoryView( me.connectionId, true );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshNotificationsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.TriggerExecutionHistory &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailable( me.connectionId ) )
    {
      var execHistoryGrid = triggersTab.down( '#execHistoryGrid' );
      // assert execHistoryGrid != null - the trigger execution history grid should exist

      // refresh button
      execHistoryGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // max fetch date picker
      execHistoryGrid.down( '#maxFetchDateSelect' ).on( 'changeMaxFetchDate',
          function ( newValue )
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // max fetch size combo
      execHistoryGrid.down( '#maxFetchSizeSelect' ).on( 'changeMaxFetchSize',
          function ( newValue )
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // event fired by the actioncolumn in the ExecHistoryGrid
      execHistoryGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      execHistoryGrid.down( '#errorColumn' ).on( 'requestViewException',
          function ( exception )
          {
            me._viewException( me.connectionId, exception );
          } );

      execHistoryGrid.down( '#resultColumn' ).on( 'requestViewResult',
          function ( execHistoryId )
          {
            me._viewResult( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#userDataColumn' ).on( 'requestViewUserData',
          function ( execHistoryId )
          {
            me._viewUserData( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#logColumn' ).on( 'requestViewLog',
          function ( execHistoryId )
          {
            me._viewLog( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        execHistoryGrid.down( '#refreshButton' ).disable();
        execHistoryGrid.down( '#maxFetchDateSelect' ).disable();
        execHistoryGrid.down( '#maxFetchSizeSelect' ).disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        execHistoryGrid.getSelectionModel().deselectAll();
      } );

      execHistoryGrid.getStore().on( 'load', function ( execHistoryStore, records, successful, operation )
      {
        var triggersGrid = triggersTab.down( '#triggersGrid' );
        var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

        if ( triggerSelected )
        {
          execHistoryGrid.down( '#refreshButton' ).enable();
          execHistoryGrid.down( '#maxFetchDateSelect' ).enable();
          execHistoryGrid.down( '#maxFetchSizeSelect' ).enable();
        }

        if ( successful )
        {
          // currently no logic
        }
        else
        {
          execHistoryStore.removeAll();

          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingExecHistoryMsg' ) );
        }
      } );
    }
  },


  /**
   * Binds the trigger statistics panel events to their handlers.
   */
  _bindEventHandlersStatisticsView : function ( triggersTab )
  {
    var me = this;

    var statisticsView = triggersTab.down( '#statisticsView' );
    statisticsView.on( 'activate', function ()
    {
      me._refreshStatisticsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshNotificationsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.TriggerStatistics &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailable( me.connectionId ) )
    {
      var statisticsPanel = statisticsView.down( '#statisticsPanel' );

      // refresh button
      statisticsPanel.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshStatisticsView( me.connectionId, true );
          } );


      // save chart button
      statisticsPanel.down( '#saveChartButton #saveChartAsJpeg' ).on( 'click',
          function ()
          {
            me._saveStatisticsChart( me.connectionId, 'image/jpeg' );
          } );

      statisticsPanel.down( '#saveChartButton #saveChartAsPng' ).on( 'click',
          function ()
          {
            me._saveStatisticsChart( me.connectionId, 'image/png' );
          } );

      statisticsPanel.down( '#saveChartButton #saveChartAsSvg' ).on( 'click',
          function ()
          {
            me._saveStatisticsChart( me.connectionId, 'image/svg+xml' );
          } );


      statisticsPanel.on( 'statisticsCriteriaChange',
          function ( statisticsType, periodLength, periodStart )
          {
            me._refreshStatisticsView( me.connectionId, true );
          } );
    }
  },


  /**
   * Binds the trigger notifications panel events to their handlers.
   */
  _bindEventHandlersNotificationsView : function ( triggersTab )
  {
    var me = this;

    var notificationsView = triggersTab.down( '#notificationsView' );
    notificationsView.on( 'activate', function ()
    {
      me._refreshNotificationsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.TriggerExecutionNotifications &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailable( me.connectionId ) )
    {
      var notificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );

      // refresh button
      notificationRulesGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          } );

      notificationRulesGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onNotificationRuleUnselected( triggersTab );
        }
        else
        {
          me._onNotificationRuleSelected( selectedRecords[0], triggersTab );
        }
      } );

      notificationRulesGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        notificationRulesGrid.down( '#refreshButton' ).disable();
        notificationRulesGrid.down( '#addNotificationRuleButton' ).disable();
        notificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
        notificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
        notificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        notificationRulesGrid.getSelectionModel().deselectAll();
      } );

      notificationRulesGrid.getStore().on( 'load', function ( notificationRuleStore, records, successful, operation )
      {
        var triggersGrid = triggersTab.down( '#triggersGrid' );
        var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

        if ( triggerSelected )
        {
          notificationRulesGrid.down( '#refreshButton' ).enable();
          notificationRulesGrid.down( '#addNotificationRuleButton' ).enable();
        }

        if ( successful )
        {
          // currently no logic
        }
        else
        {
          notificationRuleStore.removeAll();

          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingNotifRulesMsg' ) );
        }
      } );

      notificationRulesGrid.down( '#addNotificationRuleButton' ).on( 'click', function ()
      {
        var triggersGrid = triggersTab.down( '#triggersGrid' );
        var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[0];
        var trigger = triggerWithJobDetail.get( 'trigger' );

        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
          connectionId : me.connectionId,
          triggerGroupName : trigger.group.name,
          triggerName : trigger.name,
          /*
           * Upon success refresh the notifications view.
           */
          success : function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          }
        } ).execute();
      } );

      notificationRulesGrid.down( '#deleteNotificationRuleButton' ).on( 'click', function ()
      {
        var notificationRule = notificationRulesGrid.getSelectionModel().getSelection()[0];
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule', {
          connectionId : me.connectionId,
          notificationRule : notificationRule,
          /*
           * Upon success refresh the notifications view.
           */
          success : function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          }
        } ).execute();
      } );

      notificationRulesGrid.down( '#editNotificationRuleButton' ).on( 'click', function ()
      {
        var notificationRule = notificationRulesGrid.getSelectionModel().getSelection()[0];
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
          connectionId : me.connectionId,
          notificationRuleId : notificationRule.get( 'id' ),
          /*
           * Upon success refresh the notifications view.
           */
          success : function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          }
        } ).execute();
      } );

      notificationRulesGrid.down( '#cloneNotificationRuleButton' ).on( 'click', function ()
      {
        var notificationRule = notificationRulesGrid.getSelectionModel().getSelection()[0];
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
          connectionId : me.connectionId,
          notificationRuleId : notificationRule.get( 'id' ),
          /*
           * Upon success refresh the notifications view.
           */
          success : function ()
          {
            me._refreshNotificationsView( me.connectionId, true );
          }
        } ).execute();
      } );

      // opens a window with messages enqueued by the notification rule
      notificationRulesGrid.down( '#messagesColumn' ).on( 'requestViewMessages', function ( notificationRuleId )
      {
        Ext.create( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
          connectionId : me.connectionId,
          creatorId : notificationRuleId,
          creatorType : QDesk.enums.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE
        } ).execute();
      } );
    }
  },


  /**
   * Refreshes the triggers grid.
   * @param connectionId a connection ID.
   * @private
   */
  _refreshTriggersGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersGrid', {
      connectionId : connectionId
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger execution history view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshHistoryView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerHistoryView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger statistics view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshStatisticsView : function ( connectionId, populate )
  {
    var me = this;

    var triggersTab = QDesk.controller.QuartzControllerUtils.getTriggersTab( connectionId );
    var statisticsView = triggersTab.down( '#statisticsView' );

    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerStatisticsView', {
      connectionId : connectionId,
      populate : populate,
      maskEl : statisticsView.getEl()
    } ).execute();
  },


  /**
   * Invoked to save the chart in the trigger statistics view.
   * @param connectionId a connection ID.
   * @param mimeType the chart image mime type.
   */
  _saveStatisticsChart : function ( connectionId, mimeType )
  {
    var triggersTab = QDesk.controller.QuartzControllerUtils.getTriggersTab( connectionId );
    var chart = triggersTab.down( '#statisticsView #statisticsPanel #chartContainer chart' );

    Ext.create( 'QDesk.controller.command.SaveChart', {
      chart : chart,
      mimeType : mimeType
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger notifications view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshNotificationsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerExecNotificationsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked when a trigger in the TriggersGrid in the TriggerTab is selected.
   * @param triggerWithJobDetail a QDesk.model.quartz.TriggerWithJobDetail instance representing the selected trigger.
   * @param triggersTab a TriggersTab.
   */
  _onTriggerSelected : function ( triggerWithJobDetail, triggersTab )
  {
    var me = this;

    var triggersGrid = triggersTab.down( '#triggersGrid' );

    var triggerState = triggerWithJobDetail.get( 'trigger' ).state;
    switch ( triggerState )
    {
      case QDesk.enums.scheduler.quartz.TriggerState.NONE:
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.NORMAL:
        triggersGrid.down( '#resumeTriggerButton' ).disable();
        triggersGrid.down( '#pauseTriggerButton' ).enable();
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.PAUSED:
        triggersGrid.down( '#resumeTriggerButton' ).enable();
        triggersGrid.down( '#pauseTriggerButton' ).disable();
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.COMPLETE:
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.ERROR:
        break;

      case QDesk.enums.scheduler.quartz.TriggerState.BLOCKED:
        break;

      default:
        Ext.Error.raise( 'Unexpected trigger state value: ' + triggerState );
    }

    triggersGrid.down( '#deleteTriggerButton' ).enable();

    var views = triggersTab.down( '#views' );

    var connectionId = triggersTab.getConnectionId();
    var activeView = views.getActiveTab();

    me._refreshHistoryView( connectionId, activeView.itemId == 'historyView' );
    me._refreshStatisticsView( connectionId, activeView.itemId == 'statisticsView' );
    me._refreshNotificationsView( connectionId, activeView.itemId == 'notificationsView' );
  },


  /**
   * Invoked when a trigger in the TriggersGrid in the TriggersTab is unselected.
   * @param triggersTab a TriggersTab.
   */
  _onTriggerUnselected : function ( triggersTab )
  {
    var me = this;

    var triggersGrid = triggersTab.down( '#triggersGrid' );

    triggersGrid.down( '#resumeTriggerButton' ).disable();
    triggersGrid.down( '#pauseTriggerButton' ).disable();
    triggersGrid.down( '#deleteTriggerButton' ).disable();

    var connectionId = triggersTab.getConnectionId();

    me._refreshHistoryView( connectionId, false );
    me._refreshStatisticsView( connectionId, false );
    me._refreshNotificationsView( connectionId, false );
  },


  /**
   * Invoked when an execution notification rule record in the TriggerNotificationRulesGrid in the TriggerNotificationsView
   * is selected.
   * @param notificationRule a QDesk.model.quartz.ExecNotificationRule instance representing the selected
   * notification rule record.
   * @param triggersTab the TriggersTab.
   */
  _onNotificationRuleSelected : function ( notificationRule, triggersTab )
  {
    var me = this;

    var notificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );
    // assert notificationRulesGrid != null - the job notification rules grid should exist

    notificationRulesGrid.down( '#deleteNotificationRuleButton' ).enable();
    notificationRulesGrid.down( '#editNotificationRuleButton' ).enable();
    notificationRulesGrid.down( '#cloneNotificationRuleButton' ).enable();
  },


  /**
   * Invoked when a notification rule record in the TriggerNotificationRulesGrid in the TriggerNotificationsView
   * is unselected.
   * @param triggersTab the TriggersTab.
   */
  _onNotificationRuleUnselected : function ( triggersTab )
  {
    var me = this;

    var notificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );
    // assert notificationRulesGrid != null - the job notification rules grid should exist

    notificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
    notificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
    notificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();
  },


  _resumeTriggerGroup : function ( triggerGroupName )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.ResumeTriggerGroup', {
      connectionId : me.connectionId,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _pauseTriggerGroup : function ( triggerGroupName )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.PauseTriggerGroup', {
      connectionId : me.connectionId,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _resumeTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ResumeTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _pauseTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.PauseTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _deleteTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.DeleteTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  /**
   * Opens a window showing the specified job data maps.
   * @param jobDataMap a job data map.
   * @param effectiveJobDataMap an effective job data map.
   */
  _viewJobDataMap : function ( jobDataMap, effectiveJobDataMap )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
      jobDataMap : jobDataMap,
      effectiveJobDataMap : effectiveJobDataMap
    } ).execute();
  },


  /**
   * Opens a window showing the job execution exception.
   * @param connectionId a connection ID.
   * @param exception the exception data.
   */
  _viewException : function ( connectionId, exception )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.ViewError', {
      errorText : exception
    } ).execute();
  },


  /**
   * Opens a window showing the job execution log.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewLog : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryLog', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution result.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewResult : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryResult', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution user data.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewUserData : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecHistoryUserData', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  }
} );
